const mapWidth = 96;
const mapHeight = 96;
const mapArea = mapWidth * mapHeight;
const mapScavengerPlayer=7

const Tile = {
    BLUE: 0,
    ROCK: 8,
    DARK_CONCRETE_SIDE: 12,
    WATER: 17,
    DARK_CONCRETE: 18,
    CLIFF: 19,
    DIRT: 22,
    FEATURE1: 28,
    FEATURE2: 32,
    FEATURE3: 48,
    GRASS: 50,
    CONCRETE: 51,
    CRATER: 55,
};

var texture = Array(mapArea).fill(50);
var heightmap = Array(mapArea).fill(0);
var structures = [];
var droids = [];
var features = [];

function pts(x,y)
{return x+y*mapWidth}
function upts(n)
{return [n%mapWidth,Number.parseInt(n/mapWidth)]}
function urandom()
{return Math.random()}
function randint(low,high)
{return Number.parseInt(low+Math.random()*(high+.999))}


const TILE_XFLIP = 0x8000;
const TILE_YFLIP = 0x4000;
const TILE_ROT = 0x1000;
function setTexture(tex,x,y,rot,flipx) {
    if (rot===undefined)rot=randint(0,3)
    if (flipx===undefined)flipx=randint(0,1)
    texture[pts(x,y)] = tex | (rot * TILE_ROT) | (flipx*TILE_XFLIP);
}

function mapAddDroid(player, x,y, name) {
    droids.push({
        name: name,
        position: [128*x-64, 128*y-64],
        direction: 0,
        player: player
    });
}
function mapAddStructure(player, x,y, name, modules=0, direction=0) {
    structures.push({
        name: name,
        position: [128 * x, 128 * y],
        direction: 0x4000*(direction%4),
        modules: modules,
        player: player
    })
}


for (var i=0;i<mapArea;i++)
{
    var [x,y]=upts(i)
    var [ax,ay]=[((x/mapWidth)-.5)*2,((y/mapHeight)-.5)*2]
    var [epsx,epsy]=[1/mapWidth,1/mapHeight]

    var mu=(Math.abs(ay)-.3)*2
    var height=Math.min(500,Math.max(0,Number.parseInt((mu*500))))
    heightmap[i]=height

    if ((Math.abs(ay)<=epsy) && (x%4==2))mapAddStructure(mapScavengerPlayer,x,y,"X-Super-Cannon")

    if (height>4)setTexture(Tile.DIRT,x,y)
    else setTexture(Tile.FEATURE1,x,y)
}


function mBase(player,x,y,flipx=false,flipy=false)
{
    var droidTeam=[[2,0,"ConstructionDroid"],[2,1,"ConstructionDroid"],[0,2,"ConstructionDroid"],[1,2,"ConstructionDroid"]]
    
    var structTeam=[//23*12
        [0,0,"A0CommandCentre"],[3,0,"A0LightFactory",2,2],[7,0,"A0LightFactory",2,2],[11,0,"A0LightFactory",2,2],[15,0,"A0LightFactory",2,2],[19,0,"A0LightFactory",2,2],
        [0,3,"A0CyborgFactory",0,1],[3,3,"A0CyborgFactory",0,1],[6,3,"A0CyborgFactory",0,1],[9,3,"A0CyborgFactory",0,1],[12,3,"A0CyborgFactory",0,1],
        [0,6,"A0ResearchFacility",1],[3,6,"A0ResearchFacility",1],[6,6,"A0ResearchFacility",1],[9,6,"A0ResearchFacility",1],[12,6,"A0ResearchFacility",1],
        [0,9,"A0PowerGenerator",1],[3,9,"A0PowerGenerator",1],[6,9,"A0PowerGenerator",1],[9,9,"A0PowerGenerator",1],[12,9,"A0PowerGenerator",1],
        [0,12,"A0PowerGenerator",1],[3,12,"A0PowerGenerator",1],[6,12,"A0PowerGenerator",1],[9,12,"A0PowerGenerator",1],[12,12,"A0PowerGenerator",1],
        ]
    
    var [w,h]=[21,12]
    function ssize(name=""){
        if (name=="A0LightFactory")return 2
        if (name=="A0ResourceExtractor")return 2
        if (name=="A0CyborgFactory")return 1
        if (name.startsWith("A0"))return 1
        return .5
    }

    for (var i=0;i<4;i++) for (var j=0;j<10;j++){// HACK
        structTeam.push([i*2+14,j+3,"A0ResourceExtractor"])
    }
    
    droidTeam.forEach(([dx,dy,name])=>{
        mapAddDroid(player, x+dx,y+dy, name);
    })

    structTeam.forEach(([dx,dy,name,modules,rot=0])=>{
        if (flipx){dx=w-dx-ssize(name)}
        if (flipy){dy=h-dy-ssize(name); rot=rot+2}
        if (flipy && name=="A0CyborgFactory")dy=dy-1
        mapAddStructure(player, x+dx,y+dy, name, modules, rot);
    })

}


for (var y=0;y<2;y++) for (var x=0;x<3;x++)
{
    mBase(x+y*3,6+x*32,6+y*72,0,1-y%2)
}



setMapData(mapWidth, mapHeight, texture, heightmap, structures, droids, features);
